% function x=DouglasRachford(x0,prox1,prox2,gamma,nit)
%
% This function solves : 
% Min f1(x) + f2(x)
% by using a Douglas Rachford algorithm.
%
% INPUT :
% - x0 : initial point.
% - prox1 : function handle to the prox of f1 (should have 2 parameters).
% - prox2 : function handle to the prox of f2 (should have 2 parameters).
% - gamma : algorithm parameter (should be positive).
% - nit : iteration number.
%
% OUTPUT : 
% - x : the last iterate. 
%
% Developer : Pierre Weiss
function x=DouglasRachford(x0,prox1,prox2,gamma,nit,verbose)

if nargin<=5
    verbose=0;
end

y=x0;
for k=1:nit
    x=prox1(y,gamma);
    y=y+prox2(2*x-y,gamma)-x;
    
    if verbose
        if (mod(k-1,verbose)==0)
            figure(100);
            colormap gray;
            imagesc(x);
            title(sprintf('Iteration %i/%i -- Douglas-Rachford',k,nit));
        end
    end
end
