n=32;
u=double(imread('cameraman.tif'));
u=u(1:n,1:n);

%% Initialisation of the wavelet transform
addpath(genpath('./rwt-master/bin/'));
h = daubcqf(10,'min');  %Generates a wavelet filter
J = 4;  %Decomposition level

%% Generate a sampling scheme
alpha=2;
p=GenerateDistribution(size(u),alpha);
m=floor(numel(u)/5);
I=GenerateRandomSet(m,p(:));

M=zeros(size(u));
M(I)=1;
% fftshift allows displaying the low frequencies at the center.
figure(1);colormap gray;imagesc(fftshift(M));
title(sprintf('Sampling mask -- Ratio of sampled locations %1.2e',numel(I)/numel(u)));

%% Pseudo-inverse
% Sampled data
y=M.*fft2(u);
% Operator definition
Phi = @(x) fft2(midwt(x,h,J));
PhiT = @(y) midwt(ifft2(y),h,J);
%Reconstruction by pseudo-inverse
x_pseudoinverse= PhiT(y);

figure(2);colormap gray;imagesc(mdwt(x_pseudoinverse,h,J));
title('Image reconstructed from the pseudo-inverse')

%% CVX (Interior Point) reconstruction
% BEWARE : should only be tried for small images (max 64x64)
% Definition of operator A
AT=@(x) M.*fft2(midwt(x,h,J));
AAT=zeros(numel(y),numel(M));
for i=1:size(u,2)
    for j=1:size(u,1)
        eij=zeros(size(u));
        eij(j,i)=1;
        ATeij=AT(eij);
        AAT(:,j+(i-1)*size(u,2))=ATeij(:);
    end
end

%run cvx/cvx_setup
n=numel(u);
b=y(:);
cvx_begin
    variable x_cvx(n)
    minimize norm(x_cvx,1)
    subject to 
        AAT*x_cvx==b
cvx_end

x_cvx=real(reshape(x_cvx,size(u)));
figure(3);colormap gray;imagesc(midwt(x_cvx,h,J));
title('Image reconstructed from the Interior Point algorithm')

%% Douglas-Rachford
A=@(y) mdwt(real(ifft2(M.*y)),h,J);
AT=@(x) M.*fft2(midwt(x,h,J));

prox1=@(x,gamma) x+A(y-AT(x));
prox2=@(x,gamma) shrink(x,gamma);

gamma=1;
nit=1000;
x0=zeros(size(u));
x_dr=DouglasRachford(x0,prox1,prox2,gamma,nit);

figure(4);colormap gray;imagesc(midwt(x_dr,h,J));
title('Image reconstructed from the Douglas-Rachford algorithm')

%% Proximal gradient descents
x0=zeros(size(u));
sigma=10;
nit=100;
verbose=0;
tau=1/sigma;
[x_pd1,CF1]=Proximal_Descent(x0,A,AT,y,tau,sigma,nit,verbose);
[x_pd2,CF2]=Proximal_Descent_Accelerated(x0,A,AT,y,tau,sigma,nit,verbose);

figure(5);colormap gray;imagesc(midwt(x_pd1,h,J));
title('Image reconstructed from the proximal gradient descent algorithm')

figure(6);colormap gray;imagesc(midwt(x_pd2,h,J));
title('Image reconstructed from the accelerated proximal gradient descent algorithm')

figure(7);plot(1:nit,CF1,'r',1:nit,CF2,'b');
title('Cost function w.r.t. iteration number')
legend('Standard','Accelerated');

m=min(min(CF1(:)),min(CF2(:)));
figure(8);
semilogy(1:nit,CF1-m,'r',1:nit,CF2-m,'b')
title('Cost function w.r.t. iteration number - log scale')
legend('Standard','Accelerated');